#!/usr/bin/env bash

##
## list any search paths to find the 'avos' directory for each of your domains above, using a space to separate different path types
##

paths='/home/$user/html/$domain/avos /var/www/$domain/avos'

##
## leave this blank unless you need to overide the current username for user in 'paths' above
##

user=

##
## edit the path to php if different
##

if [[ -e /usr/bin/php ]];
then
	php=/usr/bin/php
elif [[ -e /usr/local/bin/php ]];
then
	php=/usr/local/bin/php
fi

function showhelp()
{
	if [[ "$1" != "" ]];
	then
		echo 
		echo "ERROR: $1"
	fi

	cat <<- HELP

	usage: [-h] [-m MODULE] [-p PATHS] [-g GROUP] [-t TIMEOUT] [domain1] [domain2] [domain3]

	TIMEOUT is how long in minutes the script is allowed to run before being killed and restarted (default = 60)

	examples:
        cron.sh
        cron.sh mydomain1.com mydomain2.com mydomain3.com
        cron.sh -m backups mydomain1.com mydomain2.com mydomain3.com
        cron.sh -m backups -t 30 mydomain1.com mydomain2.com mydomain3.com
        cron.sh -m backups -p '/home/myusername/\\\$domain/avos' mydomain1.com mydomain2.com mydomain3.com

HELP
exit
}

group=
timeout=30
maxloglines=10000
minloglines=5000

while [[ "$1" != "" ]]; 
do
	case $1 in
		-m|--module) shift; module=$1;;
		-p|--path) shift; paths=$1;;
		-g|--group) shift; group=".$1";;
		-t|--timeout) shift; timeout=$1;;
		-h) showhelp;;
		*) domains="$domains $1";;
	esac
	shift
done

if [[ "$domains" == "" ]];
then
	showhelp "you must provide a list of domains"
	exit;
fi

if [[ "$paths" == "" ]];
then
	showhelp "you must edit this file and enter the path(s) to your webroots or provide via -p on the command line"
	exit;
fi

##
## try to work out the username
##

for whoami in $USER $LOGNAME $(whoami) $(/usr/bin/whoami 2> /dev/null) $(/usr/local/bin/whoami 2> /dev/null)
do
	if [[ "$user" == "" && "$whoami" != "" ]];
	then
		user=$whoami
	fi
done

##
## debug log
##

function debug()
{
	date=$(date)
	prefix=$(printf "%s %-6s %-12s" "$date" $$ $module$group)

	echo "$prefix $1"
	echo "$prefix $1" >> /tmp/cron.$user.log
}

function get_children()
{
	ps -o pid --no-headers --ppid $1
}

function is_running()
{
	ps --no-header --ppid $1 | wc -l
}

##
## parse a command line arg
##

if [[ "$module" != "" ]];
then
	extra="$module"
	lockfile="/tmp/cron.$user$group.$module.lck"
else
	lockfile="/tmp/cron.$user$group.lck"
fi

##
## a few defines
##

cmd="$php cron.php $extra"
timeout=$(expr $timeout \* 60)

#debug "starting: module=$module lockfile=$lockfile"

##
## if we have a lock file check that it's not too old - due to a possible killed script
##

if [[ -e $lockfile ]];
then
	modifiedon=$(stat -c %Y $lockfile)
	now=$(date +%s)
	age=$(expr $now - $modifiedon)
	proc=$(cat $lockfile)
	#debug "lock file [$lockfile] exists for pid $proc, $age secs old"
	debug "lock file is $age secs old (pid $proc)"
	running=$(is_running $proc)

	if [[ $running == 0 ]];
	then
		debug "process $proc not running so removing lockfile"
		rm -f $lockfile
	else
		if [[ $modifiedon+$timeout -lt $now ]];
		then
			debug "lock is stale so removing and killing process $proc"
			children=$(get_children $proc)
			if [[ "$children" != "" ]];
			then
				debug "killing children $children of process $proc"
				kill $children
			fi
			debug "killing process $proc"
			kill $proc
			rm -f $lockfile
		else
			exit
		fi
	fi
#else
#	debug "lockfile does not exist"
fi

starttime=$(date +%s)

##
## create new lock file and try to make sure it's deleted when we exit
##

#debug "creating new lockfile [$lockfile] and running..."
debug "creating new lockfile and running..."
echo $$ > $lockfile
trap "debug \"sigint: removing lockfile $lockfile\"; rm -f $lockfile" SIGINT

for domain in $domains
do
	echo 
	echo "PROCESING [[[ $domain ]]]"
	echo

	debug "module=$module domain=$domain"
	found=0

	for path in $paths
	do
		path=$(eval echo $path)

		if [[ -e $path/bin/cron.php ]];
		then
			found=1
			export ZTOP=$path
			cd $path/bin
			$cmd
		fi
	done

	if [[ $found == 0 ]];
	then 
		error="ERROR: unable to find '$domain' via the paths provided of '$paths'"
		echo $error
	fi
done

##
## remove lockfile
##

now=$(date +%s)
runtime=$(expr $now - $starttime)
debug "removing lockfile $lockfile normally, runtime=$runtime"
rm -f $lockfile

##
## prune log file
##

lines=$(wc -l /tmp/cron.$user.log | awk '{ print $1 }')

if [[ $lines -gt $maxloglines ]]; 
then
	debug "pruning logfile ($lines)"
	tail -$minloglines /tmp/cron.$user.log > /tmp/cron.$user.log.prune
	mv /tmp/cron.$user.log.prune /tmp/cron.$user.log
fi

